import pytest
from pkg_resources import EntryPoint

from .conftest import MockNamespaceModuleManager
from worker_detector.namespace_module_manager import NamespaceModuleManager


def test_create_a_new_namespace_module_manager(mock_namespace_module_manager):
    assert mock_namespace_module_manager is not None
    assert isinstance(mock_namespace_module_manager, NamespaceModuleManager)


def test_unique_namespace_per_module_manager():
    nmm_one = NamespaceModuleManager("skidward.tests.one")
    nmm_two = NamespaceModuleManager("skidward.tests.two")

    assert nmm_one.namespace != nmm_two.namespace


def test_empty_list_of_modules_for_fake_namespace():
    fake_nmm = NamespaceModuleManager("skidward.fake.name.space")
    assert fake_nmm.get_all_available_module_names() == []


def test_modules_found_for_mocked_real_namespace(mock_namespace_module_manager):
    modules = mock_namespace_module_manager.get_all_available_module_names()
    assert isinstance(modules, list)
    assert len(modules) > 0


def test_raise_error_when_loading_non_existing_module(mock_namespace_module_manager):
    with pytest.raises(ValueError):
        mock_namespace_module_manager.load_module("fake-name")


def test_load_existing_module(mock_namespace_module_manager):
    result = mock_namespace_module_manager.load_module("real_name")
    assert result is not None
    assert isinstance(result, EntryPoint)
    assert result.module_name == "real_name"


def test_unique_list_of_modules_per_module_manager_namespace():
    nmm_first = MockNamespaceModuleManager("skidward.real.name.space")
    nmm_second = MockNamespaceModuleManager("skidward.fake.name.space")

    available_modules_first = nmm_first.get_all_available_module_names()
    available_modules_second = nmm_second.get_all_available_module_names()

    assert isinstance(available_modules_first, list)
    assert available_modules_first != available_modules_second
    assert len(available_modules_first) > len(available_modules_second)



from skidward.web import app
from flask import redirect, url_for, render_template
from flask_security import (login_required, current_user,
                            login_user, logout_user,
                            registerable)


# Creating routes
@app.route('/')
@login_required
def index():
    return render_template('index.html')


@app.route('/register')
def register():
    registerable.register_user()
    login_user()
    redirect(url_for('index.html'))


@app.route('/login')
def login():
    login_user(current_user)


@app.route('/logout')
def logout():
    logout_user(current_user)
    return redirect(url_for('index.html'))




from pathlib import Path

from dotenv import load_dotenv
import pytest

from skidward.web import app


@pytest.fixture
def init_env_vars():
    path_to_project_root = Path(__file__).parent.parent.parent
    path_to_module_env_file = (path_to_project_root / "skidward" / ".env").resolve()

    load_dotenv(dotenv_path=path_to_module_env_file, verbose=True)


def test_it_creates_app(init_env_vars):
    assert app



"""Provides an API for module detection on the provided namespace"""
from typing import Dict, Optional, List

import pkg_resources as pkg


class NamespaceModuleManager(object):
    def __init__(self, namespace: str):
        self.namespace = namespace
        self._available_modules = self._get_module_entry_points_on_namespace()

    def _get_module_entry_points_on_namespace(self) -> Dict[str, pkg.EntryPoint]:
        modules: Dict[str, pkg.EntryPoint] = {}

        for ep in pkg.iter_entry_points(self.namespace):
            modules[ep.module_name] = ep

        return modules

    def _load_module_entry_point(self, name: str) -> Optional[pkg.EntryPoint]:
        ep = self._available_modules.get(name)
        if ep:
            return ep.load()

        raise ValueError("Module with name '{}' was not found.".format(name))

    def get_all_available_module_names(self) -> List[str]:
        if not self._available_modules:
            self._available_modules = self._get_module_entry_points_on_namespace()

        return list(self._available_modules.keys())

    def load_module(self, name: str) -> pkg.EntryPoint:
        return self._load_module_entry_point(name)



from skidward.web import app
from flask_sqlalchemy import SQLAlchemy
from flask_security import UserMixin, RoleMixin, current_user, utils
from datetime import datetime as dt
from flask_admin.contrib import sqla
from wtforms.fields import PasswordField

db = SQLAlchemy(app)

# Creating Models
roles_users = db.Table('roles_users',
                       db.Column('user_id', db.Integer(), db.ForeignKey('user.id')),
                       db.Column('role_id', db.Integer(), db.ForeignKey('role.id')))


class Role(db.Model, RoleMixin):
    id = db.Column(db.Integer(), primary_key=True)
    name = db.Column(db.String(80), unique=True, default='end_user')
    description = db.Column(db.String(255))

    def __repr__(self):
        return self.name


class User(db.Model, UserMixin):
    id = db.Column(db.Integer, primary_key=True)
    email = db.Column(db.String(255), unique=True, nullable=False)
    username = db.Column(db.String(255), unique=True, nullable=False)
    password = db.Column(db.String(255), nullable=False)
    active = db.Column(db.Boolean(), default=True)
    confirmed_at = db.Column(db.DateTime(), default=dt.now())
    roles = db.relationship('Role', secondary=roles_users,
                            backref=db.backref('users', lazy='dynamic'))

    def __str__(self):
        return self.username


class Worker(db.Model, UserMixin):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(80), unique=True)
    description = db.Column(db.String(255))


class UserAdmin(sqla.ModelView):
    form_excluded_columns = ('password',)

    def is_accessible(self):
        return current_user.has_role('admin')

    def scaffold_form(self):
        form_class = super(UserAdmin, self).scaffold_form()
        form_class.password2 = PasswordField('New Password')
        return form_class

    def on_model_change(self, form, model, is_created):
        if len(model.password2):
            model.password = utils.encrypt_password(model.password2)


class RoleAdmin(sqla.ModelView):

    def is_accessible(self):
        return current_user.has_role('admin')





from setuptools import setup


setup(
    name="second-worker",
    version="0.1",
    description="Second test worker printing some text.",
    packages=["second_worker"],
    entry_points={'skidward.workers': 'second = second_worker'}
)


from setuptools import setup


setup(
    name="first-worker",
    version="0.1",
    description="First test worker printing some text.",
    packages=["first_worker"],
    entry_points={'skidward.workers': 'first = first_worker'}
)


import os

from flask import Flask
from flask_admin import Admin
from flask_admin.contrib.sqla import ModelView
from flask_security import Security, SQLAlchemyUserDatastore, utils
import click
from flask.cli import AppGroup


# Initializing the Flask App
from werkzeug.local import LocalProxy

app = Flask(__name__)

user_cli = AppGroup("user")

# Import all Models and database object after creation of app
from skidward.models import db, User, Role, Worker, UserAdmin, RoleAdmin

# Setting any FLASK_ADMIN_SWATCH(Theme Template)
app.config["FLASK_ADMIN_SWATCH"] = os.getenv("FLASK_ADMIN_SWATCH")

# Setting up Postgres Database
app.config["SQLALCHEMY_DATABASE_URI"] = os.getenv("SQLALCHEMY_DATABASE_URI")

# Setting up a secret key
app.config["SECRET_KEY"] = os.getenv("SECRET_KEY")
app.config["SECURITY_PASSWORD_SALT"] = os.getenv("SECURITY_PASSWORD_SALT")
app.config["SECURITY_REGISTERABLE"] = os.getenv("SECURITY_REGISTERABLE")
app.config["SECURITY_SEND_REGISTER_EMAIL"] = os.getenv("SECURITY_SEND_REGISTER_EMAIL")
app.config["SECURITY_PASSWORD_HASH"] = os.getenv("SECURITY_PASSWORD_HASH")

app.debug = True

# Initializing admin with flask app, name and template type
admin = Admin(app, name="Skidward-Admin", template_mode="bootstrap3")


# Setup Flask-Security
user_datastore = SQLAlchemyUserDatastore(db, User, Role)
security = Security(app, user_datastore)


@user_cli.command("create")
@click.argument("email")
def superuser(email):
    password = utils.hash_password(
        click.prompt(
            "Please enter your password", hide_input=True, confirmation_prompt=True
        )
    )
    db.create_all()
    user_datastore.find_or_create_role(name="admin", description="Administrator")
    user_datastore.find_or_create_role(name="end-user", description="End user")
    db.session.commit()

    _datastore = LocalProxy(lambda: app.extensions["security"].datastore)

    admin = _datastore.create_user(
        email=email, username=email.split("@")[0], password=password
    )
    user_datastore.add_role_to_user(email, "admin")
    db.session.commit()
    click.echo("Superuser is Created")


# Adding Models as Views to admin
admin.add_view(UserAdmin(User, db.session))
admin.add_view(RoleAdmin(Role, db.session))
admin.add_view(ModelView(Worker, db.session))


import skidward.web.views

app.cli.add_command(user_cli)

if __name__ == "__main__":
    app.run()
